@@ -21,29 +21,47 @@
   
 */
 
+#include <config.h>
+
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <string.h>
+#include <glib/gi18n.h>
 
-#include "gyrus-i18n.h"
 #include "gyrus-main-app.h"
 #include "gyrus-admin.h"
 #include "gyrus-common.h"
 #include "gyrus-session.h"
+#include "gyrus-admin-private.h"
 #include "gyrus-admin-mailbox.h"
 #include "gyrus-admin-acl.h"
 
-/* #define DEBUG_PLAIN_CHANNEL*/
+#define GYRUS_ADMIN_GET_PRIVATE(object)                           \
+	(G_TYPE_INSTANCE_GET_PRIVATE ((object), GYRUS_TYPE_ADMIN, GyrusAdminPrivate))
+
+G_DEFINE_TYPE (GyrusAdmin, gyrus_admin, GTK_TYPE_VBOX)
+
+/* #define DEBUG_PLAIN_CHANNEL */
+
+enum {
+	CONNECTED,
+	DISCONNECTED,
+	ACL_SELECTION_CHANGED,
+	MAILBOX_SELECTION_CHANGED,
+	SIGNAL_LAST
+};
+
+static gint admin_signals [SIGNAL_LAST];
 
-static void gyrus_admin_class_init (GyrusAdminClass *class);
-static void gyrus_admin_init (GyrusAdmin *admin);
 static void gyrus_admin_finalize (GObject *object);
 static void gyrus_admin_destroy (GtkObject *object);
 
-static void gyrus_admin_initialize_users_tree_view (GyrusAdmin *admin);
+static void gyrus_admin_initialize_mailbox_tree_view (GyrusAdmin *admin, 
+						      gboolean is_orphaned);
 static void gyrus_admin_initialize_acl_tree_view (GyrusAdmin *admin);
-GtkTreeModel * gyrus_admin_users_treemodel_new (void);
-GtkTreeModel * gyrus_admin_acl_treemodel_new (void);
+
+GtkTreeModel * gyrus_admin_mailbox_tree_model_new (void);
+GtkListStore * gyrus_admin_acl_tree_model_new (void);
 
 void gyrus_admin_on_button_connect_clicked (GtkButton *button,
 					    gpointer user_data);
@@ -58,12 +76,9 @@ void
 gyrus_admin_on_acl_selection_changed (GtkTreeSelection *selection,
 				      gpointer user_data);
 
-gboolean
+void
 gyrus_admin_on_users_selection_changed (GtkTreeSelection *selection,
-					GtkTreeModel *model,
-					GtkTreePath *path,
-					gboolean path_currently_selected,
-					gpointer data);
+					gpointer user_data);
 
 void gyrus_admin_on_renderer_toggled (GtkCellRendererToggle *renderer,
 				      gchar *path,
@@ -77,48 +92,74 @@ void gyrus_admin_on_acl_identifier_edited (GtkCellRendererText *cellrenderertext
 gboolean
 gyrus_admin_logout (GyrusAdmin *admin);
 
-static GtkVBoxClass *parent_class = NULL;
 #define BUFFER_SIZE 10240
 
 /*** private methods ****/
 
-GType
-gyrus_admin_get_type (void)
-{
-	static GType type = 0;
-  
-	if (!type) {
-		static const GTypeInfo info = {
-				sizeof (GyrusAdminClass),
-				NULL,           /* base_init */
-				NULL,           /* base_finalize */
-				(GClassInitFunc) gyrus_admin_class_init,
-				NULL,           /* class_finalize */
-				NULL,           /* class_data */
-				sizeof (GyrusAdmin),
-				0,              /* n_preallocs */
-				(GInstanceInitFunc) gyrus_admin_init,
-				0
-		};
-		
-		type = g_type_register_static (GTK_TYPE_VBOX,
-					       "GyrusAdmin",
-					       &info, 0);
-	}
-	
-	return type;
-}
-
 static void
 gyrus_admin_class_init (GyrusAdminClass *class)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (class);
 	GtkObjectClass *object_class = GTK_OBJECT_CLASS (class);
 
-	parent_class = g_type_class_peek_parent (class);
+	GType selection_changed_types [1] = {
+		G_TYPE_BOOLEAN
+	};
+	
+	gyrus_admin_parent_class = g_type_class_peek_parent (class);
 
 	gobject_class->finalize = gyrus_admin_finalize;
 	object_class->destroy = gyrus_admin_destroy;
+	
+	admin_signals [CONNECTED] = 
+		g_signal_newv ("connected", 
+			       G_TYPE_FROM_CLASS (class),
+			       G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+			       NULL /* class closure */,
+			       NULL /* accumulator */,
+			       NULL /* accu_data */,
+			       g_cclosure_marshal_VOID__VOID,
+			       G_TYPE_NONE /* return_type */,
+			       0     /* n_params */,
+			       NULL  /* param_types */);
+
+	admin_signals [DISCONNECTED] = 
+		g_signal_newv ("disconnected", 
+			       G_TYPE_FROM_CLASS (class),
+			       G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+			       NULL /* class closure */,
+			       NULL /* accumulator */,
+			       NULL /* accu_data */,
+			       g_cclosure_marshal_VOID__VOID,
+			       G_TYPE_NONE /* return_type */,
+			       0     /* n_params */,
+			       NULL  /* param_types */);
+
+	admin_signals [ACL_SELECTION_CHANGED] = 
+		g_signal_newv ("acl-selection-changed", 
+			       G_TYPE_FROM_CLASS (class),
+			       G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+			       NULL /* class closure */,
+			       NULL /* accumulator */,
+			       NULL /* accu_data */,
+			       g_cclosure_marshal_VOID__BOOLEAN,
+			       G_TYPE_NONE /* return_type */,
+			       1     /* n_params */,
+			       selection_changed_types /* param_types */);
+
+	admin_signals [MAILBOX_SELECTION_CHANGED] = 
+		g_signal_newv ("mailbox-selection-changed", 
+			       G_TYPE_FROM_CLASS (class),
+			       G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+			       NULL /* class closure */,
+			       NULL /* accumulator */,
+			       NULL /* accu_data */,
+			       g_cclosure_marshal_VOID__BOOLEAN,
+			       G_TYPE_NONE /* return_type */,
+			       1     /* n_params */,
+			       selection_changed_types /* param_types */);
+
+	g_type_class_add_private (class, sizeof(GyrusAdminPrivate));
 }
 
 static void
@@ -138,6 +179,9 @@ gyrus_admin_init_get_widgets (GyrusAdmin *admin)
 	priv->treeview_users =
 		GTK_TREE_VIEW (glade_xml_get_widget (xml_page,
 						     "treeview_users"));
+	priv->treeview_corrupt_mailbox = 
+		GTK_TREE_VIEW (glade_xml_get_widget (xml_page, 
+						     "treeview_corrupt_mailbox"));
 	priv->treeview_acl =
 		GTK_TREE_VIEW (glade_xml_get_widget (xml_page,
 						     "treeview_acl"));
@@ -205,12 +249,13 @@ gyrus_admin_init_connect_signals (GyrusAdmin *admin)
 static void
 gyrus_admin_init (GyrusAdmin *admin)
 {
-	admin->priv = g_new0 (GyrusAdminPrivate, 1);
+	admin->priv = GYRUS_ADMIN_GET_PRIVATE (admin);
 	admin->priv->session = NULL;
 	
 	gyrus_admin_init_get_widgets (admin);
-	gyrus_admin_initialize_users_tree_view (admin);
+	gyrus_admin_initialize_mailbox_tree_view (admin, FALSE);
 	gyrus_admin_initialize_acl_tree_view (admin);
+	gyrus_admin_initialize_mailbox_tree_view (admin, TRUE);
 	gyrus_admin_init_connect_signals (admin);
 	gyrus_admin_mailbox_set_sensitive (admin, FALSE);
 	gtk_widget_show (GTK_WIDGET (admin));
@@ -226,8 +271,8 @@ gyrus_admin_finalize (GObject *object)
 		gyrus_admin_logout (admin);
 	
 	gyrus_session_free (admin->priv->session);
-	g_free (admin->priv);
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+
+	G_OBJECT_CLASS (gyrus_admin_parent_class)->finalize (object);
 }
 
 static void
@@ -237,15 +282,15 @@ gyrus_admin_destroy (GtkObject *object)
 	g_return_if_fail (GYRUS_IS_ADMIN (object));
 	admin = GYRUS_ADMIN (object);
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	GTK_OBJECT_CLASS (gyrus_admin_parent_class)->destroy (object);
 }
 
 GtkTreeModel *
-gyrus_admin_users_tree_model_new (void)
+gyrus_admin_mailbox_tree_model_new (void)
 {
 	GtkTreeModel *model;
 
-	model = GTK_TREE_MODEL(	gtk_tree_store_new (COL_USERS_NUMBER, 
+	model = GTK_TREE_MODEL(	gtk_tree_store_new (COL_MAILBOX_NUMBER, 
 						    G_TYPE_STRING, 
 						    G_TYPE_STRING,
 						    G_TYPE_STRING));
@@ -303,20 +348,26 @@ void
 gyrus_admin_get_users_list (GyrusAdmin *admin)
 {
 	GtkTreeStore *store;
+	GtkTreeStore *store_corrupt;
 	gchar *msg;
 	gchar **current_mailbox;
 	GIOError error = G_IO_ERROR_NONE;
 	gint depth, old_depth = 1;
-	GtkTreeIter iter_new, iter_parent, iter_child;
+	GtkTreeIter iter_new, iter_parent, iter_child, iter_corrupt;
 	gboolean is_valid;
-        gint n_users;
+        gint n_users, n_orphaned;
         GtkTreeViewColumn *column;
 	GyrusImapStatus status;
 	gchar *mb_name;
+	const gchar *sep_char = gyrus_admin_get_separator_char (admin);
 	
 	store =	GTK_TREE_STORE (gtk_tree_view_get_model
 				(admin->priv->treeview_users));
+	store_corrupt = GTK_TREE_STORE (gtk_tree_view_get_model 
+					(admin->priv->treeview_corrupt_mailbox));
+
 	gtk_tree_store_clear (store);
+	gtk_tree_store_clear (store_corrupt);
 
 	msg = g_strdup (". list * *\n");
 	error = gyrus_admin_write_channel (admin, msg);
@@ -328,10 +379,25 @@ gyrus_admin_get_users_list (GyrusAdmin *admin)
 		mb_name = gyrus_admin_get_mailbox_from_list_message (msg);
 
 		current_mailbox = g_strsplit (mb_name, 
-					      gyrus_admin_get_separator_char (admin), 
+					      sep_char, 
 					      -1);
 
 		depth = 0;
+
+		/* this mailbox is fucked up */
+		if (current_mailbox [1] == NULL) {
+			/* add it somewhere */
+			gtk_tree_store_append (store_corrupt, &iter_corrupt, NULL);
+			gtk_tree_store_set (store_corrupt, &iter_corrupt,
+					    COL_MAILBOX_ICON, GTK_STOCK_DIRECTORY,
+					    COL_MAILBOX_NAME, mb_name,
+					    -1);
+			g_strfreev (current_mailbox);
+			g_free (msg);
+			g_free (mb_name);
+			continue;
+		}
+
 		is_valid = FALSE;
 			
 		while (current_mailbox [depth + 1] != NULL)
@@ -340,7 +406,7 @@ gyrus_admin_get_users_list (GyrusAdmin *admin)
 		/* current folder is at same level than previous one */
 		if (depth == old_depth) {
 			gtk_tree_store_append (store, &iter_child,
-					       (depth == 1 ?
+ 					       (depth == 1 ?
 						NULL : &iter_parent));
 			is_valid = TRUE;
 		}
@@ -372,12 +438,19 @@ gyrus_admin_get_users_list (GyrusAdmin *admin)
 
 		if (is_valid) {
 			gtk_tree_store_set (store, &iter_child,
-					    COL_USERS_ICON, GTK_STOCK_DIRECTORY,
-					    COL_USERS_MAILBOX, current_mailbox [depth],
-					    COL_USERS_MAILBOX_PATH, mb_name,
+					    COL_MAILBOX_ICON, GTK_STOCK_DIRECTORY,
+					    COL_MAILBOX_BASENAME, current_mailbox [depth],
+					    COL_MAILBOX_NAME, mb_name,
 					    -1);
 			old_depth = depth;
 		}
+		else {
+			gtk_tree_store_append (store_corrupt, &iter_corrupt, NULL);
+			gtk_tree_store_set (store_corrupt, &iter_corrupt,
+					    COL_MAILBOX_ICON, GTK_STOCK_DIRECTORY,
+					    COL_MAILBOX_NAME, mb_name,
+					    -1);
+		}
 		g_strfreev (current_mailbox);
 		g_free (msg);
 		g_free (mb_name);
@@ -393,27 +466,57 @@ gyrus_admin_get_users_list (GyrusAdmin *admin)
 	/* get the number of users in the tree */
 	n_users = gtk_tree_model_iter_n_children (GTK_TREE_MODEL(store), NULL);
 	column = gtk_tree_view_get_column (GTK_TREE_VIEW(admin->priv->treeview_users), 0);
-
 	msg = g_strdup_printf (_("Users (%d)"), n_users);
 	gtk_tree_view_column_set_title (column, msg);
 	g_free (msg);
+	
+	n_orphaned = gtk_tree_model_iter_n_children (GTK_TREE_MODEL(store_corrupt), NULL);
+	column = gtk_tree_view_get_column (GTK_TREE_VIEW(admin->priv->treeview_corrupt_mailbox), 0);
+	msg = g_strdup_printf (_("Orphaned mailboxes (%d)"), n_orphaned);
+	gtk_tree_view_column_set_title (column, msg);
+	g_free (msg);
+}
+
+gboolean
+gyrus_admin_on_treeview_focused (GtkWidget *widget,
+				 GtkDirectionType arg1,
+				 gpointer data)
+{
+	GyrusAdmin *admin = GYRUS_ADMIN (data);
+	GtkTreeSelection *selection;
+
+	g_object_set_data (G_OBJECT (admin), "selected-treeview", widget);
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
+
+	gyrus_admin_on_users_selection_changed (selection, data);
+	
+	return FALSE;
 }
 
 static void
-gyrus_admin_initialize_users_tree_view (GyrusAdmin *admin)
+gyrus_admin_initialize_mailbox_tree_view (GyrusAdmin *admin, 
+					  gboolean is_orphaned)
 {
 	g_return_if_fail (GYRUS_IS_ADMIN (admin));
-
+	
+	GtkTreeView *treeview;
+	GtkTreeModel *model;
 	GtkTreeViewColumn* column;
 	GtkCellRenderer *renderer;
-	GtkTreeModel *model;
 	GtkTreeSelection *selection;
-	
-	gtk_tree_view_set_reorderable  (admin->priv->treeview_users, FALSE);
-	
-	column = gtk_tree_view_column_new ();
-	gtk_tree_view_column_set_title (column, _("Users"));
 
+	if (is_orphaned) {
+		treeview = admin->priv->treeview_corrupt_mailbox;
+	} else {
+		treeview = admin->priv->treeview_users;
+	}
+	
+	gtk_tree_view_set_reorderable  (treeview, FALSE);
+
+	column = gtk_tree_view_column_new ();
+	gtk_tree_view_column_set_title (column, 
+					is_orphaned ? _("Orphaned mailboxes") : _("Users"));
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	g_object_set (G_OBJECT (renderer),
@@ -421,24 +524,26 @@ gyrus_admin_initialize_users_tree_view (GyrusAdmin *admin)
 	              NULL);
 	
 	gtk_tree_view_column_set_attributes (column, renderer,
-	                                     "stock-id", COL_USERS_ICON,
+	                                     "stock-id", COL_MAILBOX_ICON,
 	                                     NULL);
 	
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_set_attributes (column, renderer,
-					     "text", COL_USERS_MAILBOX,
+					     "text", is_orphaned ? COL_MAILBOX_NAME : COL_MAILBOX_BASENAME,
 					     NULL);
 
-	gtk_tree_view_append_column (admin->priv->treeview_users, column);
+	gtk_tree_view_append_column (treeview, column);
 
-	model = gyrus_admin_users_tree_model_new ();
-	gtk_tree_view_set_model (admin->priv->treeview_users, model);
+	model = gyrus_admin_mailbox_tree_model_new ();
+	gtk_tree_view_set_model (treeview, model);
 
-	selection = gtk_tree_view_get_selection (admin->priv->treeview_users);
-	gtk_tree_selection_set_select_function (selection,
-						gyrus_admin_on_users_selection_changed,
-						admin, NULL);
+	selection = gtk_tree_view_get_selection (treeview);
+	g_signal_connect (G_OBJECT (selection), "changed", 
+			  G_CALLBACK (gyrus_admin_on_users_selection_changed), admin);
+			  
+	g_signal_connect (G_OBJECT (treeview), "focus-in-event", 
+			  G_CALLBACK (gyrus_admin_on_treeview_focused), admin);
 }
 
 GtkListStore *
@@ -541,7 +646,7 @@ gyrus_admin_initialize_acl_tree_view (GyrusAdmin *admin)
 	}
 
 	selection = gtk_tree_view_get_selection (view);
-	g_signal_connect (selection, "changed",
+	g_signal_connect (G_OBJECT (selection), "changed",
 			  G_CALLBACK (gyrus_admin_on_acl_selection_changed),
 			  admin);
 	
@@ -599,7 +704,7 @@ gyrus_admin_initialize_channel (GyrusAdmin *admin)
 }
 
 /* Allocates @msg_len bytes of memory to store the message.
-   Returns the kind of message readed */ 
+   Returns the kind of message read */ 
 GyrusImapStatus
 gyrus_admin_listen_channel (GyrusAdmin *admin, gchar **message, gint *msg_len)
 {
@@ -630,7 +735,7 @@ gyrus_admin_listen_channel (GyrusAdmin *admin, gchar **message, gint *msg_len)
 		}
 		
 #ifdef DEBUG_PLAIN_CHANNEL
-		g_print ("readed:  %s", buffer);
+		g_print ("read:  %s", buffer);
 #endif
 		*message = g_strdup (buffer);
 		tokens = g_strsplit (*message, " ", 0);
@@ -698,11 +803,24 @@ gyrus_admin_logout (GyrusAdmin *admin)
 }
 
 void
-gyrus_admin_logged_out (GyrusAdmin *admin)
+gyrus_admin_clean_mailbox_treeview (GtkTreeView *treeview, const gchar *title)
 {
+	g_return_if_fail (GTK_IS_TREE_VIEW (treeview));
+
 	GtkTreeStore *store;
 	GtkTreeViewColumn *column;
 	
+	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (treeview)));
+	gtk_tree_store_clear (store);
+
+	column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), 0);
+        gtk_tree_view_column_set_title (column, title);
+	
+}
+
+void
+gyrus_admin_logged_out (GyrusAdmin *admin)
+{
 	GtkWidget *btn_image;
 	
 	gtk_button_set_label (GTK_BUTTON (admin->priv->button_connect),
@@ -712,26 +830,59 @@ gyrus_admin_logged_out (GyrusAdmin *admin)
 	gtk_button_set_image (GTK_BUTTON (admin->priv->button_connect),
 			      btn_image);
 
-	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (admin->priv->treeview_users)));
-	gtk_tree_store_clear (store);
-	column = gtk_tree_view_get_column(GTK_TREE_VIEW (admin->priv->treeview_users), 0);
-        gtk_tree_view_column_set_title (column, _("Users"));
+	gyrus_admin_clean_mailbox_treeview (GTK_TREE_VIEW (admin->priv->treeview_users), 
+					    _("Users"));
+	gyrus_admin_clean_mailbox_treeview (GTK_TREE_VIEW (admin->priv->treeview_corrupt_mailbox), 
+					    _("Orphaned mailboxes"));
 
 	gyrus_admin_mailbox_clear_info (admin);
 	gyrus_admin_mailbox_set_sensitive (admin, FALSE);
 	
-	if (GYRUS_IS_MAIN_APP (admin->priv->parent_app)) {
-		gyrus_main_app_menu_set_sensitive (GYRUS_MAIN_APP
-						   (admin->priv->parent_app),
-						   FALSE);
-		gyrus_main_app_menu_item_set_sensitive
-			(GYRUS_MAIN_APP (admin->priv->parent_app),
-			 "/commands/EditMailboxRemove", FALSE);
-		gyrus_main_app_menu_item_set_sensitive
-			(GYRUS_MAIN_APP (admin->priv->parent_app),
-			 "/commands/AclEntryNew", FALSE);
+	g_signal_emit (admin, admin_signals [ACL_SELECTION_CHANGED],
+		       0,
+		       NULL);
+	g_signal_emit (admin, admin_signals [DISCONNECTED],
+		       0, NULL);
+	
+}
 
+/* This function was taken from camel.
+   Copyright 2000 Ximian, Inc. (www.ximian.com)
+*/
+/**
+ * imap_quote_string:
+ * @str: the string to quote, which must not contain CR or LF
+ *
+ * Return value: an IMAP "quoted" corresponding to the string, which
+ * the caller must free.
+ **/
+char *
+imap_quote_string (const char *str)
+{
+	const char *p;
+	char *quoted, *q;
+	int len;
+	
+	g_assert (strchr (str, '\r') == NULL);
+	
+	len = strlen (str);
+	p = str;
+	while ((p = strpbrk (p, "\"\\"))) {
+		len++;
+		p++;
 	}
+	
+	quoted = q = g_malloc (len + 3);
+	*q++ = '"';
+	for (p = str; *p; ) {
+		if (strchr ("\"\\", *p))
+			*q++ = '\\';
+		*q++ = *p++;
+	}
+	*q++ = '"';
+	*q = '\0';
+	
+	return quoted;
 }
 
 /* returns whether the conection can be established or not. */
@@ -740,6 +891,7 @@ gyrus_admin_login (GyrusAdmin *admin)
 {
 	GTcpSocket *socket;
 	gchar *msg;
+	gchar *q_password;
 	GyrusImapStatus status;
 	GyrusAdminLoginError error = GYRUS_ADMIN_LOGIN_OK;
 	socket = gyrus_admin_create_socket_from_session (admin->priv->session);
@@ -754,10 +906,12 @@ gyrus_admin_login (GyrusAdmin *admin)
 	g_free (msg);
 
 	g_return_val_if_fail (status == GYRUS_IMAP_STATUS_OK, FALSE);
-	
+
+	q_password = imap_quote_string (gyrus_admin_get_current_passwd (admin));
 	msg = g_strdup_printf (". login \"%s\" %s\n",
 			       gyrus_admin_get_current_user (admin),
-			       gyrus_admin_get_current_passwd (admin));
+			       q_password);
+	g_free (q_password);
 	gyrus_admin_write_channel (admin, msg);
 	g_free (msg);
 
@@ -788,13 +942,12 @@ gyrus_admin_login (GyrusAdmin *admin)
 void
 gyrus_admin_post_login (GyrusAdmin *admin)
 {
-	if (GYRUS_IS_MAIN_APP (admin->priv->parent_app)) {
-		gyrus_main_app_menu_set_sensitive (GYRUS_MAIN_APP
-						   (admin->priv->parent_app),
-						   TRUE);
-	}
 	if (strcmp (gyrus_admin_get_separator_char (admin), "auto") == 0)
 		gyrus_admin_set_separator_char_auto (admin);
+
+	g_signal_emit (admin, admin_signals [CONNECTED],
+		       0,
+		       NULL);
 	
 	gyrus_admin_get_users_list (admin);
 }
@@ -844,7 +997,8 @@ gyrus_admin_pre_login (GyrusAdmin *admin)
 				GtkWidget *btn_image;
 				gtk_button_set_label (GTK_BUTTON (admin->priv->button_connect),
 						      _("_Disconnect"));
-				btn_image = gtk_image_new_from_stock (GTK_STOCK_DISCONNECT, GTK_ICON_SIZE_BUTTON);
+				btn_image = gtk_image_new_from_stock (GTK_STOCK_DISCONNECT, 
+								      GTK_ICON_SIZE_BUTTON);
 				gtk_button_set_image (GTK_BUTTON (admin->priv->button_connect),
 						      btn_image);
 				gyrus_admin_post_login (admin);
@@ -866,7 +1020,7 @@ gyrus_admin_select_foreach_func (GtkTreeModel *model, GtkTreePath *path,
 	
 	gchar *this_mailbox;
 	gtk_tree_model_get (model, iter,
-			    COL_USERS_MAILBOX_PATH, &this_mailbox,
+			    COL_MAILBOX_NAME, &this_mailbox,
 			    -1);
 
 	if (!strcmp (gtk_label_get_text (GTK_LABEL (admin->priv->label_mailbox_name)),
@@ -902,29 +1056,29 @@ gchar *
 gyrus_admin_get_selected_mailbox (GyrusAdmin *admin)
 {
 	GtkTreeSelection *selection;
+	GtkTreeView *treeview;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	
 	gchar *current_mailbox = NULL;
 	
-	selection = gtk_tree_view_get_selection
-		(GTK_TREE_VIEW (admin->priv->treeview_users));
+	treeview = GTK_TREE_VIEW (g_object_get_data (G_OBJECT (admin), 
+						     "selected-treeview"));
+	/* case we haven't already set it */
+	if (treeview == NULL)
+		return NULL;
+
+	selection = gtk_tree_view_get_selection	(treeview);
+
 	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
 
 		gtk_tree_model_get (model, &iter,
-				    COL_USERS_MAILBOX_PATH, &current_mailbox,
+				    COL_MAILBOX_NAME, &current_mailbox,
 				    -1);
 	}
 	return current_mailbox;
 }
 
-void
-gyrus_admin_set_parent_app (GyrusAdmin *admin,
-			    gpointer app)
-{
-	admin->priv->parent_app = app;
-}
-
 gboolean
 gyrus_admin_is_connected (GyrusAdmin *admin)
 {
@@ -1041,6 +1195,14 @@ gyrus_admin_refresh_users_list (GyrusAdmin *admin)
 	gyrus_admin_get_users_list (admin);
 }
 
+GtkTreeView *
+gyrus_admin_get_users_treeview (GyrusAdmin *admin)
+{
+	g_return_val_if_fail (GYRUS_IS_ADMIN (admin), NULL);
+
+	return g_object_ref (admin->priv->treeview_users);
+}
+
 GtkWidget *
 gyrus_admin_new (GyrusSession *session)
 {
@@ -1062,10 +1224,6 @@ gyrus_admin_on_button_connect_clicked (GtkButton *button,
 
 	if (gyrus_admin_is_connected (admin)) {
 		gyrus_admin_pre_logout (admin);
-		if (GYRUS_IS_MAIN_APP (admin->priv->parent_app))
-			gyrus_main_app_menu_set_sensitive (GYRUS_MAIN_APP
-							   (admin->priv->parent_app),
-							   FALSE);
 	} else {
 		gyrus_admin_pre_login (admin);
 	}
@@ -1088,41 +1246,27 @@ gyrus_admin_on_acl_selection_changed (GtkTreeSelection *selection,
 	GtkTreeModel *model;
 	GtkTreeIter  iter;
 
-	if (gtk_tree_selection_get_selected (selection, &model, &iter))	{
-		gyrus_main_app_menu_item_set_sensitive
-			(GYRUS_MAIN_APP (admin->priv->parent_app),
-			 "/commands/AclEntryDelete", TRUE);
-		gyrus_main_app_menu_item_set_sensitive
-			(GYRUS_MAIN_APP (admin->priv->parent_app),
-			 "/commands/AclEntryRename", TRUE);
-				
-	} else {
-		gyrus_main_app_menu_item_set_sensitive
-			(GYRUS_MAIN_APP (admin->priv->parent_app),
-			 "/commands/AclEntryDelete", FALSE);
-		gyrus_main_app_menu_item_set_sensitive
-			(GYRUS_MAIN_APP (admin->priv->parent_app),
-			 "/commands/AclEntryRename", FALSE);
-	}
+	g_signal_emit (admin, admin_signals [ACL_SELECTION_CHANGED],
+		       0,
+		       gtk_tree_selection_get_selected (selection, &model, &iter),
+		       NULL);
 }
 
-gboolean
+void 
 gyrus_admin_on_users_selection_changed (GtkTreeSelection *selection,
-					GtkTreeModel *model,
-					GtkTreePath *path,
-					gboolean path_currently_selected,
-					gpointer data)
+					gpointer          user_data) 
 {
+
 	GtkTreeIter iter;
-	GyrusAdmin *admin = GYRUS_ADMIN (data);
-	gchar *mailbox_path;
-	
-	g_return_val_if_fail (gtk_tree_model_get_iter(model, &iter, path),
-			      TRUE);
-	
-	if (!path_currently_selected) {
+	GtkTreeModel *model;
+	GyrusAdmin *admin = GYRUS_ADMIN (user_data);
+
+	gboolean is_selected = gtk_tree_selection_get_selected (selection, &model, &iter);
+	if (is_selected)
+	{
+		gchar *mailbox_path;
 		gtk_tree_model_get (model, &iter,
-				    COL_USERS_MAILBOX_PATH, &mailbox_path,
+				    COL_MAILBOX_NAME, &mailbox_path,
 				    -1);
 
 		gchar **tokens;
@@ -1130,30 +1274,19 @@ gyrus_admin_on_users_selection_changed (GtkTreeSelection *selection,
 		gyrus_admin_mailbox_show_info (admin, tokens[1],
 					       mailbox_path);
 		g_strfreev (tokens);
-		
-		gyrus_admin_mailbox_set_sensitive (admin, TRUE);
-		if (GYRUS_IS_MAIN_APP (admin->priv->parent_app)) {
-			gyrus_main_app_menu_item_set_sensitive
-				(GYRUS_MAIN_APP (admin->priv->parent_app),
-				 "/commands/EditMailboxRemove", TRUE);
-		}
-		
 		g_free (mailbox_path);
+		gyrus_admin_mailbox_set_sensitive (admin, TRUE);
+		
 	}
 	else {
 		gyrus_admin_mailbox_clear_info (admin);
 		gyrus_admin_mailbox_set_sensitive (admin, FALSE);
-		if (GYRUS_IS_MAIN_APP (admin->priv->parent_app)) {
-			gyrus_main_app_menu_item_set_sensitive
-				(GYRUS_MAIN_APP (admin->priv->parent_app),
-				 "/commands/EditMailboxRemove", FALSE);
-			gyrus_main_app_menu_item_set_sensitive
-				(GYRUS_MAIN_APP (admin->priv->parent_app),
-				 "/commands/AclEntryNew", FALSE);
-		}
 	}
 
-	return TRUE;
+	g_signal_emit (admin, admin_signals [MAILBOX_SELECTION_CHANGED],
+		       0,
+		       is_selected,
+		       NULL);	
 }
 
 /***************************************
@@ -1328,7 +1461,7 @@ gyrus_admin_on_acl_identifier_edited (GtkCellRendererText *cellrenderertext,
 					    -1);
 		}
 	} else if (gyrus_admin_ask_user_if_overwrite (admin, newname)) {
-		/* DELET EXISTING ONE */
+		/* DELETE EXISTING ONE */
 		if (!gyrus_admin_acl_delete_entry (admin, mailbox,
 						   newname, &error)) {
 			g_warning (error);
